import TinyQueue from 'tinyqueue';
function defaultComparator(a, b) {
  return a < b ? -1 : a > b ? 1 : 0;
}
export default function merge(arrays, options) {
  let comparator,
    outputMetadata = false;
  if (typeof options === 'function') {
    comparator = options;
  } else if (options) {
    comparator = options.comparator;
    outputMetadata = options.outputMetadata;
  }
  const finalComparator = comparator || defaultComparator;
  function entryComparator(a, b) {
    return finalComparator(a[2], b[2]);
  }
  const totalLength = arrays.reduce(function (length, array) {
    return length + array.length;
  }, 0);
  const output = new Array(totalLength);
  let outputIndex = 0;
  const initQueue = arrays.reduce(function (initQueue, array, index) {
    if (array.length) initQueue.push([index, 0, array[0]]);
    return initQueue;
  }, []);
  const queue = new TinyQueue(initQueue, entryComparator);
  let entry;
  while (entry = queue.pop()) {
    output[outputIndex++] = outputMetadata ? entry : entry[2];
    const array = arrays[entry[0]];
    const nextIndex = entry[1] + 1;
    if (nextIndex < array.length) {
      queue.push([entry[0], nextIndex, array[nextIndex]]);
    }
  }
  return output;
}
//# sourceMappingURL=index.mjs.map